/*
  File: include/acl_util.h
  (Linux Access Control List Management)

  Last modified: Wed 22 Sep 1999, 23:40

  Copyright (C) 1999 by Andreas Gruenbacher
  <a.gruenbacher@computer.org>
 	
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef __ACL_UTIL_H
#define __ACL_UTIL_H

#include <sys/types.h>  /* for uid_t and gid_t */
#include <stdio.h>  /* for FILE */


#ifdef __cplusplus
extern "C" {
#endif


typedef struct {
	int entries;
	int size;
	acl_entry_t *entp;
} acl_t;


/* manip.c */
acl_t *
acl_init(
        int entries);
int
acl_reserve(
        acl_t *aclp,
	int entries);
acl_t *
acl_dup(
        const acl_t *aclp);
int
acl_free(
        acl_t *aclp);
int
acl_create_entry(
        acl_t *aclp,
        acl_entry_t **entp);
int
acl_delete_entry(
        acl_t *aclp,
        acl_entry_t *entp);
int
acl_copy_entry(
        acl_entry_t *destp,
        const acl_entry_t *srcp);
acl_entry_t *
acl_iterate(
        acl_t *aclp,
        acl_entry_t *entp);
void
acl_entry_sort(
	acl_entry_t *aclentp,
	int aclcnt);
int
acl_sort(
	acl_t *aclp);
int
acl_check(
        acl_t *aclp,
        int *whichp);
const char *
acl_strerror(
        int code);
char *
acl_to_text(
        const acl_t *aclp);
acl_t *
acl_from_text(
        const char *textp);
acl_t *
acl_get_file(
	const char *pathp,
	int type);
int
acl_find(
	acl_t *aclp,
	acl_entry_t **user_obj,
	acl_entry_t **group_obj,
	acl_entry_t **mask_obj,
	acl_entry_t **other_obj);


/* usr_grp.c */
int
parse_user(
	const char **endp,
	 uid_t *uid);
const char *
user_name(
	uid_t uid);
int
parse_group(
	const char **endp,
	 gid_t *gid);
const char *
group_name(
	gid_t gid);

#define ACL_PARSE_WITH_PERM	(0x0001)
#define ACL_PARSE_NO_PERM	(0x0002)
#define ACL_PARSE_PERM		(0x0001 | 0x0002)

#define ACL_PARSE_WITH_RELATIVE	(0x0004)
#define ACL_PARSE_NO_RELATIVE	(0x0008)
#define ACL_PARSE_RELATIVE	(0x0004 | 0x0008)

#define ACL_PARSE_MULTI         (0x0010)

/* acl_text.c */
int
parse_aclent(
	const char *textp,
	const char **endp,
	acl_entry_t *acl,
	int parse_mode);
acl_t *
parse_acl(
	const char *textp,
	const char **which,
	int parse_mode);
int
acl_entry_to_str(
	const acl_entry_t *aclent,
	char *textp,
	size_t size);


/* libacl.c */
/*int
aclsort2(
	acl_entry_t *aclbufp,
	int aclcnt);*/
/*int acllookup(
	acl_entry_t *aclbufp,
	int aclcnt,
	acl_entry_t **user_obj,
	acl_entry_t **group_obj,
	acl_entry_t **class_obj,
	acl_entry_t **other_obj);*/


/* flags for acl_print()'s what parameter */
#define ACL_PRINT_ACCESS_ACL	(1)
#define ACL_PRINT_DEFAULT_ACL	(2)
#define ACL_PRINT_EFFECTIVE	(4)
#define ACL_PRINT_ALL_EFFECTIVE	(8)

/* acl_io.c */
int
acl_print(
	FILE *file,
	const acl_t *aclp,
	int what);
acl_t *
acl_read(
	FILE *file,
	int parse_mode,
	int *line,
	int *which,
	char **filename);
/*const char *
tabs(
	int max,
	const char *str);*/


/* acl type flags returned by acl_check_one() */
#define ACL_HAS_MASK		(0x0002)
#define ACL_HAS_BIT_ERROR	(0x8000)

/* additional type flags returned by acl_check() */
#define ACL_HAS_ACCESS_ACL	(0x0001)
#define ACL_HAS_DEFAULT_ACL	(0x1000)
#define ACL_HAS_DEFAULT_MASK	(0x2000)

/* flags for `command sequence' ACLs */
#define ACL_SEQ_CMD		(0xFC0000)  /* all command bits combined */
#define ACL_SEQ_REPLACE		(0x000000)
#define ACL_SEQ_ADD		(0x040000)
#define ACL_SEQ_SUBTRACT	(0x080000)
#define ACL_SEQ_REMOVE_ENTRY	(0x100000)
#define ACL_SEQ_REMOVE_DEFAULT  (0x200000)
#define ACL_SEQ_REMOVE_EXTENDED (0x400000)
#define ACL_SEQ_REMOVE_ALL      (0x800000)

/* acl_kernel.c */

int acl_check_one(
        const acl_entry_t *aclbufp,
        int aclcnt,
        const acl_entry_t **last,
        int type_flag);
int
acl_entries_equiv_mode(
	const acl_entry_t *aclentp,
	int aclcnt,
	mode_t *modep);
int
acl_equiv_mode(
	const acl_t *aclp,
	mode_t *modep);
void
mode_to_acl_entries(
	int mode,
	uid_t uid,
	gid_t gid,
	acl_entry_t *aclentp);
int
mode_to_acl(
	const struct stat *buf,
	acl_t *aclp);

/*
int aclcheck2(const acl_entry_t *aclentp, int aclcnt, int *which);
int acleqmode(const acl_entry_t *aclbufp, int aclcnt, mode_t *modep);
int modetoacl(const struct stat *buf, acl_entry_t *aclbufp, int aclcnt);
int acl_masq(acl_entry_t *aclbufp, int aclcnt, mode_t *mode, mode_t mask) ;
int acl_chmod(acl_entry_t *aclbufp, int aclcnt, mode_t mode);
const char *aclstrerror(int error);
acl_entry_t *acl_file(acl_entry_t *aclbufp, int aclcnt, int *aclfilecnt);
acl_entry_t *acl_default(acl_entry_t *aclbufp, int aclcnt, int *acldefcnt);
*/

#ifdef __cplusplus
}
#endif

#endif

