/*
  File: include/sys/acl.h
  (Linux Access Control List Management)

  Last modified: Wed 22 Sep 1999, 23:40

  Copyright (C) 1999 by Andreas Gruenbacher
  <a.gruenbacher@computer.org>
 	
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef __SYS_ACL_H
#define __SYS_ACL_H

#include <sys/types.h>
#include <sys/stat.h>
#include <stddef.h>

#ifdef __cplusplus
"C" {
#endif


typedef struct {
	int	a_type;
	uid_t	a_id;		/* may be UID | GID | ACL_UNDEFINED_ID */
	mode_t	a_perm;
} acl_entry_t;


#define ACL_UNDEFINED_ID	((uid_t)0xFFFF)

/* a_type entry in acl_entry_t */
#define ACL_UNDEFINED_TAG       (0x00)
#define ACL_USER_OBJ		(0x01)
#define ACL_USER		(0x02)
#define ACL_GROUP_OBJ		(0x04)
#define ACL_GROUP		(0x08)
#define ACL_MASK_OBJ		(0x10)
#define ACL_CLASS_OBJ		ACL_MASK_OBJ
#define ACL_OTHER_OBJ		(0x20)

#define ACL_TYPE_ACCESS		(0x2000)  /* not or'ed to the entry type! */
#define ACL_TYPE_DEFAULT	(0x1000)

#define ACL_DEF_USER_OBJ	(ACL_TYPE_DEFAULT | ACL_USER_OBJ)
#define ACL_DEF_USER		(ACL_TYPE_DEFAULT | ACL_USER)
#define ACL_DEF_GROUP_OBJ	(ACL_TYPE_DEFAULT | ACL_GROUP_OBJ)
#define ACL_DEF_GROUP		(ACL_TYPE_DEFAULT | ACL_GROUP)
#define ACL_DEF_MASK_OBJ	(ACL_TYPE_DEFAULT | ACL_MASK_OBJ)
#define ACL_DEF_CLASS_OBJ	ACL_DEF_MASK_OBJ
#define ACL_DEF_OTHER_OBJ	(ACL_TYPE_DEFAULT | ACL_OTHER_OBJ)

/* cmd arg to acl() / facl() */
#define ACL_GETACL		(1)
#define ACL_SETACL		(2)
#define ACL_GETACLCNT		(3)


/* acl_check error codes */
/* error codes returned by aclcheck() */
#define ACL_MULTI_ERROR         (0x100)     /* multiple unique objects */
#define ACL_DUPLICATE_ERROR     (0x200)     /* duplicate Id's in entries */
#define ACL_MISS_ERROR          (0x300)     /* missing required entry */
#define ACL_ENTRY_ERROR         (0x400)     /* wrong entry type */


int acl(
	const char *pathp,
	int cmd,
	int aclcnt,
	acl_entry_t *aclbufp);
int facl(
	int filedes,
	int cmd,
	int aclcnt,
	acl_entry_t *aclbufp);

#ifdef __cplusplus
}
#endif

#endif

