/*
  File: lib/usr_grp.c
  (Linux Access Control List Management)

  Last modified: Wed 22 Sep 1999, 23:40

  Copyright (C) 1999 by Andreas Gruenbacher
  <a.gruenbacher@computer.org>
 	
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <pwd.h>
#include <grp.h>

#include <sys/acl.h>
#include "acl_util.h"


int try_numeric(const char **endp, uid_t *id)
{
	const char *ep;
	long l;

	l = strtoul(*endp, (char **)&ep, 0);
	if (**endp == '-') {
		/*
			Negative values are interpreted as 16-bit numbers,
			so that uid -2 maps to nobody, etc.
		*/
		l &= 0xFFFF;
	}

	if (*endp != ep) {
		if (l == (uid_t)-1)
			return EOF;  /* overflow */

		*endp = ep;
		*id = (uid_t)l;

		return 0;
	}
	return EOF;
}


int parse_user(const char **endp, uid_t *uid)
{
	char tmp;
	const char *ep;
	struct passwd *passwd;

	ep = *endp;
	while (*ep > 32 && *ep != ':' && *ep != ',') ep++;

	if (ep == *endp) {
		*uid = ACL_UNDEFINED_ID;
		return EOF;
	} else if (try_numeric(endp, uid) == 0) {
		return 0;
	} else {
		tmp = *ep;
		*(char *)ep = '\0';  /* cast away const */
		passwd = getpwnam(*endp);
		*(char *)ep = tmp;

		if (passwd == NULL)
			return EOF;
		*uid = passwd->pw_uid;
	  *endp = ep;
	}
	return 0;
}

const char *user_name(uid_t uid)
{
	struct passwd *passwd = getpwuid(uid);

	if (passwd != NULL)
		return passwd->pw_name;
	else
		return NULL;
}


int parse_group(const char **endp, gid_t *gid)
{
	char tmp;
	const char *ep;
	struct group *group;

	ep = *endp;
	while (*ep > 32 && *ep != ':' && *ep != ',') ep++;

	if (ep == *endp) {
		*gid = ACL_UNDEFINED_ID;
		return EOF;
	} else if (try_numeric(endp, gid) == 0) {
		return 0;
	} else {
		tmp = *ep;
		*(char *)ep = '\0';  /* cast away const */
		group = getgrnam(*endp);
		*(char *)ep = tmp;

	  if (group == NULL)
	  	return EOF;
	  *gid = group->gr_gid;
	  *endp = ep;
	}
	return 0;
}


const char *group_name(gid_t gid)
{
	struct group *group = getgrgid(gid);

	if (group != NULL)
		return group->gr_name;
	else
		return NULL;
}

