/*
  File: test/acl.c
  (Linux Access Control List Management)

  Copyright (C) 1999 by Andreas Gruenbacher
  <a.gruenbacher@computer.org>
 	
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/acl.h>

#include "acl_util.h"


void acldump(const char *msg, aclent_t *acl, int aclcnt);
void aclprint2(const char *msg, aclent_t *acl, int aclcnt);


int main(int argc, char *argv[])
{
	int which;
	aclent_t *acl;
	int	aclcnt;
	int argi = 1;
	int calc_mask;
	int check;

	if (argi < argc && strcmp(argv[argi], "-m") == 0)
		calc_mask = 1;
	else
		calc_mask = 0;

	for (; argi<argc; argi++) {
		if (strcmp(argv[argi], "-") == 0) {
			acl = aclread(stdin, &aclcnt, 0, NULL);
			if (acl == NULL) {
				printf("readacl() failed.\n");
				continue;
			}
		} else {
			acl = aclfromtext(argv[argi], &aclcnt);
			if (acl == NULL) {
				printf("aclfromtext() failed.\n");
				continue;
			}
		}

		acldump("Dump:  ", acl, aclcnt);
		aclprint2("       ", acl, aclcnt);
		check = aclcheck2(acl, aclcnt, &which);
		if (check < 0) {
			printf("aclcheck2() failed at entry %d: %s.\n",
				which, aclstrerror(-check));
		} else {
			aclprint2("Check: ", acl, aclcnt);
			if (aclsort(aclcnt, calc_mask, acl) != 0) {
				printf("aclsort() + calc_class failed.\n");
			} else {
				aclprint2("Mask:  ", acl, aclcnt);
			}
			printf("\n");
			aclprint(stdout, acl, aclcnt, ACL_PRINT_ACL | ACL_PRINT_DEF_ACL |
			                              ACL_PRINT_ALL_EFFECTIVE);
		}
		free(acl);
	}

	return 0;
}


void acldump(const char *msg, aclent_t *acl, int aclcnt) {
	int m;

	printf("%s", msg);
	for (m=0; m<aclcnt; m++) {
		printf("0x%hX:0x%hX:0%o ", acl[m].a_type,
					 acl[m].a_id, acl[m].a_perm);
	}
	printf("\n");
}

void aclprint2(const char *msg, aclent_t *acl, int aclcnt) {
	char *text = acltotext(acl, aclcnt);
	if (text) {
		printf("%s%s\n", msg, text);
		free(text);
	} else {
		printf("acltotext() failed.\n");
	}
}

